# -*- coding: utf-8 -*-

__title__ = 'drf-jwt'
__version__ = '1.13.4'
__author__ = 'José Padilla, Styria Digital'
__license__ = 'MIT'
__copyright__ = '2014-2017 Blimp LLC, 2018 Styria Digital'

# Version synonym
VERSION = __version__
